package com.unclouded.android;

import java.io.Serializable;


/**
 * Describes an object that acts as a proxy for a result that is initially unknown.
 * 
 * <p>A {@link Promise} is returned as the result of a remote method invocation on a 
 * {@link RemoteReference}. Because remote method invocations are processed 
 * asynchronously (i.e. non blocking), a promise object is returned while awaiting for 
 * the return value. When this value is eventually returned, the {@link Promise} object is 
 * said to be resolved.</p>
 * 
 * <p>A {@link Promise} object accepts one or more  {@link PromiseListener PromiseListeners} 
 * that are triggered when the promise is resolved.</p>
 *
 */
public class Promise {
	
	/**
	 * Install a {@link PromiseListener} to obtain the return value 
	 * when the promise is resolved.
	 * 
	 * @param listener  used to trigger a callback method when the promise is resolved.
	 */
	public void when(PromiseListener<? extends Serializable> listener){};
	
	
	/**
	 * Returns a string representation of a Promise object. 
	 * 
	 * @return a string representation of the promise.
	 */	
	public String toString(){ return "[Promise]"; }

}
