package com.unclouded.android;

import java.io.Serializable;

/**
 * Monitors the state of a promise.
 * 
 * <p>The {@link #isResolved(Serializable) isResolved} callback method 
 * is triggered when the corresponding {@link Promise} is resolved.</p>
 *
 * @param <T>  the type of the return value that is expected to be resolved. 
 *             It should implement {@link Serializable} because return values 
 *             are passed by copy.
 */

public abstract class PromiseListener<T extends Serializable> {
	
	
	/**
	 * Triggered when the return value of the promise is resolved.
	 * @param value  the resolved return value.
	 */
	public abstract void isResolved(T value);
	
	
	/**
	 * Returns a string representation of a PromiseListener object. 
	 * 
	 * @return a string representation of the promise listener.
	 */	
	public String toString(){ return "[PromiseListener]"; }

}
