package com.unclouded.android;

import java.io.Serializable;

/**
 * Provides a reference to an object hosted by another device.
 * 
 * <p>Remote references are returned upon discovering a service 
 * referencing a {@link UObject}. To interact with this object,
 * remote references only allow asynchronous method invocations. 
 * These are invocations that are non-blocking and immediately 
 * return a {@link Promise} object.</p>
 *
 */
public class RemoteReference {
	
	/**
	 * Asynchronously invoke a method of the referenced object.
	 * 
	 * @param selector  the name of the method to invoke.
	 * @param args  additional arguments of the remote method invocation. 
	 * Arguments are passed by copy and should therefore be {@link Serializable}.
	 * @return a promise object awaiting the return value to be resolved.
	 */
	public Promise asyncInvoke(String selector, Serializable... args){
		return new Promise();
	}
	
	/**
	 * Returns a string representation of a RemoteReference object. 
	 * 
	 * @return a string representation of the remote reference.
	 */	
	public String toString() { return "[RemoteReference]"; }
	
	
	/**
	 * Indicates whether some object is equal to this remote reference.
	 * 
	 *  @param obj  the object with which to compare.
	 *  @return true if this remote reference is the same as the obj argument; false otherwise.
	 */
	public boolean equals(Object o) { return false; }

	
	/**
	 * Returns a hash code value for the remote reference.
	 * 
	 * @return a hash code value for this remote reference.
	 */
	public int hashCode() { return 0; }

	
}
