package com.unclouded.android;

import java.io.Serializable;

/**
 * Identifies services during service publication or subscription. 
 * Type tags are used to filter services from the pool of services in the network. 
 * 
 * <p>A {@link TypeTag} instance provides no extra functionality rather than serving 
 * as a classification mechanism for services.</p>
 *
 */
public class TypeTag implements Serializable {
	
	private static final long serialVersionUID = 8513789930791345943L;
	
	private String tag;
	
	/**
	 * Construct a type tag by passing a {@link String} as identifier.
	 * 
	 * @param tag  a string to identify the type tag.
	 */
	public TypeTag(String tag) {
		this.tag = tag;
	}

	
	/**
	 * Return the String identifier of the type tag.
	 * 
	 * @return a string identifier.
	 */
	public String getTag() {
		return tag;
	}
	
	
	/**
	 * Returns a string representation of the type tag. 
	 * 
	 * @return a string representation of the type tag.
	 */
	public String toString() { return "TypeTag["+this.getTag()+"]"; };
	
	
	/**
	 *  Indicates whether some other object is equal to this type tag.
	 *  
	 *  <p>Two type tags are equal when their tags are equal.<p>
	 *  
	 *  @param obj  the object with which to compare.
	 *  @return true if this type tag is the same as the obj argument; false otherwise.
	 */
	public boolean equals(Object obj) {
	      return ((TypeTag)obj).getTag().equals(this.tag);
	 }
	
	
	/**
	 * Returns a hash code value for the type tag.
	 * 
	 * @return a hash code value for this type tag.
	 */
	public int hashCode() {
	    return (tag != null) ? tag.hashCode() : 0;
	}

}
