/**
 * AmbientTalk/2 Project
 * CMDProvideService.java created on 22-feb-2007 at 20:24:31
 * (c) Programming Technology Lab, 2006 - 2007
 * Authors: Tom Van Cutsem & Stijn Mostinckx
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */
package com.unclouded.android.cmd;

import com.unclouded.android.TypeTag;
import com.unclouded.android.comm.Address;
import com.unclouded.android.comm.CommunicationBus;
import com.unclouded.android.ids.ServiceId;
import com.unclouded.android.internal.InternalEventLoop;


/**
 * A CMDProvideService message is sent asynchronously to all connected VMs when
 * an actor on the sender VM provided a new publication. Connected VMs are notified
 * of this such that they can match this new publication with local subscriptions. 
 * 
 * SENDER: the VM that provides a new publication
 * RECEIVER: all connected VMs
 * MODE: ASYNCHRONOUS, MULTICAST
 * PROPERTIES: the subscription info: topic and provided service object
 * REPLY: none
 * 
 * @author tvcutsem
 */
public class CMDCancelService extends Command {
	
	private static final long serialVersionUID = -1205984459736801225L;

	private final ServiceId objectId_;
	private final TypeTag service_;
	
	public CMDCancelService(TypeTag service, ServiceId objectId) {
		super("provideService");
		objectId_ = objectId;
		service_ = service;
	}
	
	public void send(CommunicationBus dispatcher) {
		//Logging.TEST_LOG.debug("Cancel request sent for" + service_);
		dispatcher.sendAsyncMulticast(this);
	}
	
	/**
	 * When a remote actor receives the notification, it checks whether any subscriptions
	 * can be matched with the service to cancel.
	 */
	public void uponReceiptBy(InternalEventLoop remoteHost, Address senderAddress) {
		remoteHost.event_serviceCanceled(service_, objectId_);
	}
	
}