/**
 * AmbientTalk/2 Project
 * CMDHandshake.java created on 22-feb-2007 at 20:26:21
 * (c) Programming Technology Lab, 2006 - 2007
 * Authors: Tom Van Cutsem & Stijn Mostinckx
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */
package com.unclouded.android.cmd;

import com.unclouded.android.comm.Address;
import com.unclouded.android.comm.CommunicationBus;
import com.unclouded.android.ids.EventLoopId;
import com.unclouded.android.internal.InternalEventLoop;


/**
 * A handshake command is sent when a actor discovers another actor in its environment.
 * It sends its own ActorId to the newly discovered actor, which can then store this actorId.
 *
 * SENDER: the discovering VM
 * RECEIVER: the discovered VM
 * MODE: ASYNCHRONOUS, UNICAST
 * PROPERTIES: VirtualMachineID of sender
 * REPLY: none
 * 
 * @author tvcutsem
 */
public class CMDInitializeConnection extends Command {

	private static final long serialVersionUID = 941583671647354717L;
	
	private final EventLoopId senderVMId_;
	
	public CMDInitializeConnection(EventLoopId senderVMId) {
		super("handshake");
		senderVMId_ = senderVMId;
	}
	
	public void send(CommunicationBus dispatcher, Address recipientActor) {
		dispatcher.sendAsyncUnicast(this, recipientActor);
		//Logging.ActorCommand_LOG.debug("Handshake sent");
	}
	
	public void uponReceiptBy(InternalEventLoop remoteHost, Address senderAddress) {
		//Logging.ActorCommand_LOG.debug("Handshake received");
		remoteHost.event_memberJoined(senderAddress, senderVMId_);
	}
	
}
