/**
 * AmbientTalk/2 Project
 * CMDTransmitATMessage.java created on 22-feb-2007 at 14:23:11
 * (c) Programming Technology Lab, 2006 - 2007
 * Authors: Tom Van Cutsem & Stijn Mostinckx
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */
package com.unclouded.android.cmd;

import com.unclouded.android.comm.Address;
import com.unclouded.android.comm.CommunicationBus;
import com.unclouded.android.internal.InternalEventLoop;
import com.unclouded.android.logging.Logging;
import com.unclouded.android.remoteref.Letter;

/**
 * A CMDTransmitATMessage message encapsulates an AmbientTalk message being sent
 * from a sending actor to a recipient actor.
 * 
 * SENDER: a remote far reference
 * RECEIVER: the VM hosting the object that the far reference denotes
 * MODE: SYNCHRONOUS, UNICAST
 * PROPERTIES:
 *  - Packet representing serialized AT message,
 *  - id of the actor that should unserialize and process the message
 * REPLY: no other VM command, but should return synchronous acknowledgement
 * 
 * @author tvcutsem
 */
public class CMDTransmitLetter extends Command {

	private static final long serialVersionUID = -1369124457059610846L;
	
	private final Letter letter;
	
	public CMDTransmitLetter(Letter letter_) {
		super("transmitLetter("+letter_+")");
		letter = letter_;
	}
	
	public void send(CommunicationBus dispatcher, Address recipientActor) {
		Logging.ActorCommand_LOG.debug("Letter Command transmitted");
		dispatcher.sendSynchronousUnicast(this, recipientActor);
	}
	
	public void uponReceiptBy(InternalEventLoop remoteHost, Address senderAddress) {
		Logging.ActorCommand_LOG.debug("Letter Command received");
		remoteHost.event_letterReceived(letter, senderAddress);
	}
}