package com.unclouded.android.remoteref;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import com.unclouded.android.eventloop.Event;
import com.unclouded.android.ids.EventLoopId;
import com.unclouded.android.internal.InternalEventLoop;
import com.unclouded.android.services.RemoteObjectService;

public class RemoteReferencesThreadPool {
	
	private final InternalEventLoop hostActor;
	
	/** the pool of threads**/
	private final ExecutorService pool;
	
	public RemoteReferencesThreadPool(InternalEventLoop hostActor_){
		hostActor = hostActor_;
		pool = Executors.newCachedThreadPool();
	}
	
	protected final void receive(Event event) {
		Handle h = new Handle(event);
	   	pool.execute(h);
	}
	
	
	/** transmitting is embedded first in another Event so that only 
	  * a thread from this thread pool will dequeue messages from the mailbox.
	  */
	@SuppressWarnings("serial")
	public void event_serve(final RemoteObjectService remoteObjectService) {
		receive(new Event("Serve letter") {
			public void process(Object owner) {
				try {
					synchronized (this) {
						Letter nextLetter = remoteObjectService.serveLetter();
						if(nextLetter != null){
							EventLoopId remoteActorId = remoteObjectService.getServiceId().getActorId();
							hostActor.sendLetter(nextLetter, remoteActorId);
						}
					}	
				} catch (Exception e) {	}
			}
		});
	}

	
	
	class Handle implements Runnable{
		private Event event_;
		public Handle( Event event){
			event_ = event;
		}
		public void run() {
			event_.process(hostActor);
		}	
	}
	

}
