package com.unclouded.android.services;

import java.io.Serializable;
import java.util.LinkedList;

import com.unclouded.android.Promise;
import com.unclouded.android.RemoteReference;
import com.unclouded.android.ids.PromiseId;
import com.unclouded.android.ids.ServiceId;
import com.unclouded.android.internal.InternalPromise;
import com.unclouded.android.internal.InternalRemoteReference;
import com.unclouded.android.internal.InternalEventLoop;
import com.unclouded.android.remoteref.Letter;

public class RemoteObjectService extends RemoteService {
	
	/**
     *  'outbox' stores all messages sent to the receiver object hosted by another actor.
	 *  It contains objects that implement the {@link Letter} interface.
	 *  
     */
    protected LinkedList<Letter> outbox = new LinkedList<Letter>();
	
	private final InternalRemoteReference remoteReference;

	public RemoteObjectService(ServiceId serviceId) {
		super(serviceId);
		remoteReference = new InternalRemoteReference(this);
	}
	
	public Promise asyncInvoke(String selector, Serializable... args){
		InternalPromise promise = new InternalPromise();
		PromiseId promiseId = promise.getPromiseId();
		Letter letter = new Letter(promiseId, serviceId, selector, args);
		synchronized(this){ outbox.addLast(letter); }
		InternalEventLoop.getInstance().registerAsyncInvocation(this, promiseId, promise);
		return (Promise) promise;
	}
	
	public Letter serveLetter(){
		synchronized(this) {
			if (outbox.size() > 0){
				Letter next = outbox.removeFirst();
				return next;
			}
		}
		return null;
	}

	public Object getPublicValue() {
		return (RemoteReference) remoteReference;
	}
	

}
